import sys
import json
import requests
import datetime

class Slims():

    def __init__(self, url, username, pwd):
        self.url = url
        self.username = username
        self.pwd = pwd

    def get_attachment(self, proj, exp, step, attm, active, linked, output):
        """Download a file from a slims experiment attachment step."""

        if active is None:
            active = "true"
        active = active.lower()
        if linked is None:
            linked = "true"
        linked = linked.lower()
        if output is None:
            output = attm

        # Retrieve Project
        criteria = [
            {"fieldName":"xprn_name",
            "operator":"equals",
            "value":exp},
            {"fieldName":"user_userName",
            "operator":"equals",
            "value":self.username}
        ]

        if proj:
            project = requests.get(self.url + "/Project/advanced",
                auth=(self.username, self.pwd),
                headers={"Content-Type":"application/json"},
                json={"criteria":{"operator":"and",
                        "criteria":[
                            {"fieldName":"prjc_name",
                            "operator":"equals",
                            "value":proj},
                            {"fieldName":"user_userName",
                            "operator":"equals",
                            "value":self.username}
                        ]
                    }
                }
            )

            if len(project.json()["entities"]) > 1:
                sys.exit("Multiple projects found with name '" + proj +
                    "'. Make sure projects names are unique."
                )
            elif len(project.json()["entities"]) == 0:
                sys.exit("No project found with name '" + proj + "'.")

            criteria.append({"fieldName":"xprn_fk_project",
                "operator":"equals",
                "value":project.json()["entities"][0]["pk"]}
            )

        experiment_run = requests.get(self.url + "/ExperimentRun/advanced",
            auth=(self.username, self.pwd),
            headers={"Content-Type":"application/json"},
            json={"criteria":{
                    "operator":"and",
                    "criteria":criteria
                }
            }
        )
    
        if len(experiment_run.json()["entities"]) > 1:
            sys.exit("Multiple experiments found with name '" + exp +
                "'. Make sure experiments names are unique."
            )
        elif len(experiment_run.json()["entities"]) == 0:
            sys.exit("No experiment found with name '" + exp + "'.")

        # Retrieve ExperimentRunStep
        criteria = [
            {"fieldName":"xprs_fk_experimentRun",
            "operator":"equals",
            "value":experiment_run.json()['entities'][0]['pk']},
            {"fieldName":"xpst_type",
            "operator":"equals",
            "value":"ATTACHMENT_STEP"},
            {"fieldName":"xpst_name",
            "operator":"equals",
            "value":step}
        ]

        if active != "both":
            criteria.append({"fieldName":"xprs_active",
                "operator":"equals",
                "value":active})

        experiment_step = requests.get(self.url + "/ExperimentRunStep/advanced",
            auth=(self.username, self.pwd),
            headers={"Content-Type":"application/json"},
            json={"criteria":{"operator":"and",
                    "criteria":criteria
                }
            }
        )

        if len(experiment_step.json()["entities"]) > 1:
            sys.exit("Multiple steps found with name '" + step +
                "' in experiment '" + exp +
                "'. Make sure steps names are unique."
            )
        elif len(experiment_step.json()["entities"]) == 0:
            sys.exit("No step found with name '" + step +
                "' in experiment '" + exp + "'."
            )

        # Retrieve Attachment
        criteria = [
            {"fieldName":"attm_name",
            "operator":"equals",
            "value":attm},
            {"fieldName":"user_userName",
            "operator":"equals",
            "value":self.username}
        ]

        if linked != "both":
            criteria.append({"fieldName":"attm_linkCount",
                "operator":"greaterThan",
                "value":0})
        
        attachment = requests.get(self.url + "/Attachment/advanced",
            auth=(self.username, self.pwd),
            headers={"Content-Type":"application/json"},
            json={"criteria":{"operator":"and",
                    "criteria":criteria
                }
            }
        )

        attachment = [
            [{"attm_file_filename":e1["value"], "pk":e0["pk"]}
                for e1 in e0["columns"]
                if e1["name"] == "attm_file_filename"
            ][0]
            for e0 in attachment.json()["entities"]
        ]

        attachment_link = requests.get(self.url + "/AttachmentLink/advanced",
            auth=(self.username, self.pwd),
            headers={"Content-Type":"application/json"},
            json={"criteria":{"operator":"and",
                    "criteria":[
                        {"fieldName":"atln_recordTable",
                        "operator":"equals",
                        "value":"ExperimentRunStep"},
                        {"fieldName":"atln_recordPk",
                        "operator":"equals",
                        "value":experiment_step.json()['entities'][0]['pk']}
                    ]
                }
            }
        )

        attachment_link_pk = [
            e2[0] for e2 in [[e1["value"]
                    for e1 in e0["columns"]
                    if e1["name"] == "atln_fk_attachment"
                    and (e1["value"] in [e2["pk"] for e2 in attachment])
                ]
                for e0 in attachment_link.json()["entities"]
            ]
            if e2
        ]

        if len(attachment_link_pk) > 1:
            sys.exit("Multiple attachments found with name '" + attm +
                "' in experiment '" + exp +
                "' step '" + step +
                "'. Make sure attachments names are unique."
            )
        elif len(attachment_link_pk) == 0:
            sys.exit("No attachment found with name '" + attm +
                "' in experiment '" + exp +
                "' step '" + step + "'."
                )

        # Download the attachement
        with requests.get(self.url + "/repo/" + str(attachment_link_pk[0]),
            auth=(self.username, self.pwd),
            stream=True) as r:
            r.raise_for_status()
            with open(output, "wb") as f:
                for chunk in r.iter_content(chunk_size=8192):
                    if chunk:
                        f.write(chunk)

        # Save metadata
        metadata = {"url":self.url + "/repo/" + str(attachment_link_pk[0]),
            "creator":self.username,
            "project_name":proj,
            "experiment_name":exp,
            "step_name":step,
            "attachment_name":attm,
            "attachment_file_filename":[
                e["attm_file_filename"]
                for e in attachment
                if e["pk"] == attachment_link_pk[0]
            ][0],
            "file_name":output,
            "created":datetime.datetime.now(
                datetime.timezone.utc
            ).isoformat(sep='T')
        }

        with open(output + "_metadata.txt", "w") as f:
            json.dump(metadata, f, indent=2)



from setuptools import find_packages, setup

setup(
    name = 'slims-lisp',
    version = '0.0.7',
    description = 'A high-level CLI for Slims REST API',
    long_description = open('README.rst').read(),
    long_description_content_type = 'text/x-rst',
    license = 'Apache License 2.0',
    author = 'Laboratory of Integrative System Physiology (LISP) at EPFL',
    author_email = 'alexis.rapin@epfl.ch',
    url = 'https://github.com/auwerxlab/slims-lisp-python-api',
    download_url = 'https://github.com/auwerxlab/slims-lisp-python-api/archive/v0.0.7.tar.gz',
    packages = find_packages(),
    install_requires = [
        'click',
        'requests',
        'datetime',
    ],
    entry_points = {
        'console_scripts': [
            'slims-lisp = slims_lisp.__main__:cli'
        ]
    },
)



import click
from slims_lisp.slims import Slims

@click.group()
def cli():
    pass

@cli.command(help="Download a file and its associated metadata from a slims experiment attachment step.")
@click.option('--url',
    help = 'Slims REST URL. ex: https://<your_slims_address>/rest/rest',
    required = True)
@click.option('--proj',
    help = 'Project name (if any).',
    required = False)
@click.option('-e', '--exp',
    help = 'Experiment name.',
    required = True)
@click.option('-s', '--step',
    default = 'data_collection',
    help = 'Experiment step name.',
    required = True,
    show_default = True)
@click.option('-a', '--attm',
    help = 'Attachment name.',
    required = True)
@click.option('--active',
    help = 'Search only in active or inactive steps (or in both).',
    type = click.Choice(['true', 'false', 'both'],
        case_sensitive = False),
    default = 'true',
    required = False,
    show_default = True)
@click.option('-l', '--linked',
    help = 'Search only linked or unlinked attachments (or both).',
    type = click.Choice(['true', 'false', 'both'],
        case_sensitive = False),
    default = 'true',
    required = False,
    show_default = True)
@click.option('-o', '--output',
    help = 'Output file name. [default: same as --attm]',
    required = False)
@click.option('-u', '--username',
    prompt = "User",
    help = 'User name (prompted).',
    required = True)
@click.option('-p', '--pwd',
    prompt = "Password",
    hide_input = True,
    help = 'Password (prompted).',
    required = True)
def get(url, username, pwd, proj, exp, step, attm, active, linked, output):
    slims = Slims(url = url,
        username = username,
        pwd = pwd)
    slims.get_attachment(proj = proj,
        exp = exp,
        step = step,
        attm = attm,
        active = active,
        linked = linked,
        output = output
    )

if __name__ == '__main__':
    cli()
